"""Error handling utilities for Google Search MCP Server."""

import logging
from typing import Optional
from urllib.error import URLError, HTTPError
from requests.exceptions import RequestException, ConnectionError, Timeout
from .formatter import format_error_response


# Set up logging
logger = logging.getLogger(__name__)


def handle_search_error(error: Exception, query: Optional[str] = None) -> str:
    """Handle search-related errors and return user-friendly messages.
    
    Args:
        error: The exception that occurred
        query: The search query that caused the error (for logging)
        
    Returns:
        Formatted error message for the user
    """
    # Log the error for debugging
    if query:
        logger.error(f"Search error for query '{query}': {type(error).__name__}: {str(error)}")
    else:
        logger.error(f"Search error: {type(error).__name__}: {str(error)}")
    
    # Handle specific error types
    if isinstance(error, ConnectionError):
        return format_error_response(
            "Network connection failed. Please check your internet connection and try again."
        )
    
    if isinstance(error, Timeout):
        return format_error_response(
            "Search request timed out. Please try again later."
        )
    
    if isinstance(error, HTTPError):
        if error.response and error.response.status_code == 429:
            return format_error_response(
                "Rate limit exceeded. Please wait a moment and try again."
            )
        elif error.response and error.response.status_code >= 500:
            return format_error_response(
                "Google search service is temporarily unavailable. Please try again later."
            )
        else:
            return format_error_response(
                f"HTTP error occurred: {error.response.status_code if error.response else 'Unknown'}"
            )
    
    if isinstance(error, URLError):
        return format_error_response(
            "Network error occurred. Please check your internet connection."
        )
    
    if isinstance(error, RequestException):
        return format_error_response(
            "Network request failed. Please try again later."
        )
    
    # Handle validation errors
    if isinstance(error, ValueError):
        return format_error_response(str(error))
    
    # Handle generic exceptions
    error_msg = str(error)
    if "rate limit" in error_msg.lower() or "too many requests" in error_msg.lower():
        return format_error_response(
            "Rate limit exceeded. Please wait a moment and try again."
        )
    
    if "network" in error_msg.lower() or "connection" in error_msg.lower():
        return format_error_response(
            "Network connection issue. Please check your internet connection and try again."
        )
    
    # Generic error message
    return format_error_response(
        f"Search failed: {error_msg}"
    )


def handle_validation_error(error: ValueError) -> str:
    """Handle validation errors specifically.
    
    Args:
        error: The validation error
        
    Returns:
        Formatted validation error message
    """
    logger.warning(f"Validation error: {str(error)}")
    return format_error_response(str(error))


def handle_tool_error(tool_name: str, error: Exception) -> str:
    """Handle tool execution errors.
    
    Args:
        tool_name: Name of the tool that failed
        error: The exception that occurred
        
    Returns:
        Formatted error message
    """
    logger.error(f"Tool '{tool_name}' error: {type(error).__name__}: {str(error)}")
    
    if isinstance(error, ValueError):
        return handle_validation_error(error)
    
    return format_error_response(
        f"Tool execution failed: {str(error)}"
    )


def log_error(message: str, error: Optional[Exception] = None) -> None:
    """Log an error message with optional exception details.
    
    Args:
        message: The error message to log
        error: Optional exception to include in the log
    """
    if error:
        logger.error(f"{message}: {type(error).__name__}: {str(error)}")
    else:
        logger.error(message)