"""Result formatting utilities for Google Search MCP Server."""

from typing import List
from .models import SearchResult


def format_search_results(query: str, results: List[SearchResult]) -> str:
    """Format search results into a readable text response.
    
    Args:
        query: The original search query
        results: List of SearchResult objects
        
    Returns:
        Formatted string with numbered search results
    """
    if not results:
        return f"No search results found for query: '{query}'"
    
    # Create header
    result_count = len(results)
    header = f"Search Results for '{query}' ({result_count} result{'s' if result_count != 1 else ''}):\n\n"
    
    # Format each result
    formatted_results = []
    for i, result in enumerate(results, 1):
        formatted_result = result.to_formatted_string(i)
        formatted_results.append(formatted_result)
    
    # Combine header and results
    return header + "\n\n".join(formatted_results)


def format_error_response(error_message: str) -> str:
    """Format an error message for MCP response.
    
    Args:
        error_message: The error message to format
        
    Returns:
        Formatted error response
    """
    return f"Error: {error_message}"


def format_validation_error(field: str, issue: str) -> str:
    """Format a validation error message.
    
    Args:
        field: The field that failed validation
        issue: Description of the validation issue
        
    Returns:
        Formatted validation error message
    """
    return f"Validation Error - {field}: {issue}"