"""Google Search MCP Server implementation."""

import asyncio
import logging
from mcp.server.models import InitializationOptions
import mcp.types as types
from mcp.server import NotificationOptions, Server
import mcp.server.stdio

from .search_handler import execute_google_search
from .validation import validate_google_search_arguments, validate_tool_name
from .formatter import format_search_results
from .error_handler import handle_search_error, handle_validation_error, handle_tool_error

# Set up logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# Create MCP server instance
server = Server("mcp-server-googlesearch")


@server.list_tools()
async def handle_list_tools() -> list[types.Tool]:
    """
    List available tools.
    Returns the google_search tool with its JSON Schema validation.
    """
    return [
        types.Tool(
            name="google_search",
            description="Perform a Google search and return structured results",
            inputSchema={
                "type": "object",
                "properties": {
                    "query": {
                        "type": "string",
                        "description": "The search query to execute"
                    },
                    "num_results": {
                        "type": "integer",
                        "description": "Number of search results to return (default: 10)",
                        "minimum": 1,
                        "maximum": 50,
                        "default": 10
                    }
                },
                "required": ["query"]
            },
        )
    ]


@server.call_tool()
async def handle_call_tool(
    name: str, arguments: dict | None
) -> list[types.TextContent | types.ImageContent | types.EmbeddedResource]:
    """
    Handle tool execution requests.
    Processes google_search requests and returns formatted results.
    """
    try:
        # Validate tool name
        validate_tool_name(name)
        
        # Validate arguments
        query, num_results = validate_google_search_arguments(arguments or {})
        
        logger.info(f"Executing Google search: query='{query}', num_results={num_results}")
        
        # Execute search
        search_results = await execute_google_search(query, num_results)
        
        # Format results
        formatted_response = format_search_results(query, search_results)
        
        logger.info(f"Search completed successfully: {len(search_results)} results found")
        
        return [
            types.TextContent(
                type="text",
                text=formatted_response,
            )
        ]
        
    except ValueError as e:
        # Handle validation errors
        error_response = handle_validation_error(e)
        return [
            types.TextContent(
                type="text",
                text=error_response,
            )
        ]
        
    except Exception as e:
        # Handle all other errors
        query_for_logging = arguments.get("query") if arguments else None
        error_response = handle_search_error(e, query_for_logging)
        return [
            types.TextContent(
                type="text",
                text=error_response,
            )
        ]


async def main():
    """Main entry point for the MCP server."""
    logger.info("Starting Google Search MCP Server")
    
    # Run the server using stdin/stdout streams
    async with mcp.server.stdio.stdio_server() as (read_stream, write_stream):
        await server.run(
            read_stream,
            write_stream,
            InitializationOptions(
                server_name="mcp-server-googlesearch",
                server_version="0.1.0",
                capabilities=server.get_capabilities(
                    notification_options=NotificationOptions(),
                    experimental_capabilities={},
                ),
            ),
        )